package audiotesting;

import be.hogent.tarsos.dsp.AudioDispatcher;
import java.util.ArrayList;
import javax.swing.JPanel;

/**
 * Clase encargada de definir la funcionalidades basica de un bloque
 */
public abstract class Bloque implements UpdateListener, InSignal, OutSignal{
  
    private final int id;
    private ArrayList <InSignal> drains;

    protected Bloque (int nextId){
        id = nextId;
        drains = new ArrayList<>();
    }

    public abstract void computeOutput();
    
    public abstract AudioDispatcher getDispatcher();
    
    public abstract JPanel getPanel();
    
    public void propagateOutputValue(){
        for (InSignal drain: drains) 
            drain.inputValueChanged();
    }
    public abstract void inputValueChanged();   
    
    public void connectOutput(InSignal drain) {
        if (drain != null) {
            drains.add(drain); 
            drain.addSource(this);
        }
    }
    
    public void connectDrain(InSignal drain) {
        if (drain!=null) drains.add(drain);
    }
    
    public void removeFromDrains(InSignal drain) {
        if (drain!=null) drains.remove(drain);
    }
    
    public void disconnectOutput(InSignal drain) {
        if (drain != null){
            drains.remove(drain);
            drain.removeFromSource(this);
        }
    }

    public void disconnectAllOutput() {
        while(drains.size()!=0) {
            drains.get(0).removeFromSource(this);
            drains.remove(0);
        }
    }
    
    public String getDescription () {
        return getClass().getSimpleName()+"_"+id;
    }
    
} 