package audiotesting;

import be.hogent.tarsos.dsp.AudioDispatcher;
import be.hogent.tarsos.dsp.filters.HighPass;
import java.util.ArrayList;
import javax.swing.JPanel;

/**
 * Clase Encargada de Ejecutar las funciones de un filtro digital pasa alto.
 */

public class FiltroPasaalto extends Bloque{
  
  private ArrayList<OutSignal> sources;
  private static int nextId = 0;
  private AudioDispatcher dispatcher;
  private int FreqCorte = 3000;
  private PPasaalto panel;
    
  /**
   * Constructor
   */
    public FiltroPasaalto(){
        super(nextId++);    
        sources= new ArrayList<OutSignal>();
        panel = new PPasaalto(this);
    }

    public void connectInput(OutSignal output){
        sources.add(output);
        output.connectDrain(this);
    }
    
    public void disconnectInput(){
        while(!sources.isEmpty()) {
            sources.get(0).removeFromDrains(this);
            sources.remove(0);
        }
    }
    
    public void addSource(OutSignal output) {
        sources.add(output);
    }
    
    public void removeFromSource(OutSignal output){
        sources.remove(output);
    }
    
    /**
     * Verifica si tiene conectada alguna entrada
     * @return 
     */
    public boolean hasConnectedSource() {
        return sources.size()>0;
    }

    @Override
    public void computeOutput() {
        dispatcher.addAudioProcessor(new HighPass(FreqCorte/10, dispatcher.getFormat().getSampleRate()));
        System.out.println("Frecuancia de Corte: "+FreqCorte);
        this.propagateOutputValue();
    }

    @Override
    public AudioDispatcher getDispatcher() {
        return dispatcher;
    }


    @Override
    public void inputValueChanged() {
         dispatcher = sources.get(0).getDispatcher();
         this.computeOutput();
         this.propagateOutputValue();
    }


  @Override
  public JPanel getPanel() {
        return panel;
  }
  
  /**
   * Setea el parametro de la nueva frecuencuia de corte del filtro
   * @param newfreq 
   */
  public void setFreqCorte(int newfreq){
      FreqCorte = newfreq;
  }  
  
}