package audiotesting;

import java.awt.*;
import java.awt.geom.*;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import javax.imageio.ImageIO;
/**
 * Clase encargada de asignar las propiedades graficas al bloque AudioSalida
 */
class GAudioSalida extends AudioSalida implements Drawable, GInSignal  {
    private static final Path2D.Double polyline = 
                       new Path2D.Double(Path2D.WIND_EVEN_ODD,10);
    private static final Rectangle2D.Double input= new Rectangle2D.Double(-20, 30, 10, 10);
    static {  // static initialization block. 
        polyline.moveTo(-10, 35);
        polyline.lineTo(10, 35);
    }
    private Color color = Color.BLACK;
    private Path2D.Double shape;
    private Path2D.Double in;
    private Path2D.Double out;
    private final BasicStroke stroke;
    private BufferedImage image;
    private Path2D.Double imageshape;
    
    /**
     * Constructor
     * @param pos
     * @param sim 
     */
    public GAudioSalida(Point2D.Double pos, Simulator sim){
        super(sim);
        openimage();
        AffineTransform at = AffineTransform.getTranslateInstance(pos.x, pos.y);
        shape = (Path2D.Double) at.createTransformedShape(polyline);
        in = (Path2D.Double) at.createTransformedShape(input);
        stroke = new BasicStroke(2);
        imageshape = (Path2D.Double) at.createTransformedShape(
                new Rectangle2D.Double(0,0,80,70));
    }

    public void draw (Graphics2D g){
        g.setColor(Color.BLACK);
        g.setStroke(stroke);
        g.draw(shape);
        g.setColor(color);
        g.setStroke(stroke);
        g.drawImage(image, (int) imageshape.getBounds2D().getX(), (int) imageshape.getBounds2D().getY()-5, null);
        //g.draw(imageshape);
        if (hasConnectedDrains())
            g.fill(in);
        else
            g.draw(in);
     }
    
    public boolean contains (Point p){
        return imageshape.contains(p);
    }

    public void dragIn(int dx, int dy) {
       AffineTransform at = AffineTransform.getTranslateInstance(dx, dy);
       shape.transform(at);
       in.transform(at);
       imageshape.transform(at);
       disconnectInput();
    }
    public void flip() {
        AffineTransform af = new AffineTransform();   
        Rectangle2D box = imageshape.getBounds2D(); 
        double x = box.getX()+box.getWidth()/2;

        // Note: the transformations get applied IN REVERSE ORDER!
        af.translate(x,0);
        af.scale(-1, 1);
        af.translate(-x, 0);          
        shape.transform(af);
        in.transform(af);
        
        AffineTransform tx = AffineTransform.getScaleInstance(-1, 1);
        tx.translate(-image.getWidth(null), 0);
        AffineTransformOp op = new AffineTransformOp(tx, AffineTransformOp.TYPE_NEAREST_NEIGHBOR);
        image = op.filter(image, null);
    }
    
    public void setSelected() {
        color = Color.RED;
    }
    
    public void setReleased(){
        color = Color.BLACK;
    }
    
    public Shape getGInput() {
        return in;
    }
    
    public Shape getGOutput() {
        return null;
    }
    
    /**
     * Metodo encargado de abrir la imagen correspondiente al bloque
     */
    public void openimage(){
        try {
            image = ImageIO.read(new File("./Images/AudioOut80x80v2.png"));
        } catch (IOException e) {
                e.printStackTrace();
        }
   }

    public Bloque getBloque(){
       return null;
    }
    
    public void removeDraw(){
        disconnectInput();
    }
    
}